@@ -5,23 +5,28 @@ Each grammar type can both parse string containing a value of a given type, and
 """
 
 import pyparsing as pp
+ppc = pp.pyparsing_common
+from pyparsing import Word, Suppress
+
 import io
 import inspect
-from pyparsing import Word, Suppress
 import itertools
 import numpy as np
 from collections import namedtuple
 from collections.abc import Hashable
-from .misc import OrderedDict, cached_property, cache, add_to_signature
-ppc = pp.pyparsing_common
+from .misc import OrderedDict
+from .decorators import cached_class_property, cached_property, cache, \
+                        add_to_signature, add_called_class_as_argument
 from .grammar import generate_grammar, separator as separator_grammar, \
                      delimitedList, line_end, optional_quote,\
                      replace_whitechars, White
+from .alternative_types import normalize_type, allowed_types
 
 from ase.units import Rydberg
 import copy
 import datetime
-from typing import Union, Any, Callable
+import functools
+from typing import Union, Any, Callable, Optional, Type, Dict, List
 
 context =  generate_grammar()
 context.__enter__()
@@ -37,7 +42,7 @@ def compare_numpy_values(a,b):
 #will be initialized later
 type_from_type_map = {}
 
-def type_from_type(type, format='', format_all=False):
+def type_from_type(type, format:Union[str,Dict]='', format_all:bool=False, type_map:Dict={}):
   """ Guess and return the grammar element (GrammarType class descendatnt) from a python type. E.g. int => Integer.
 
       The given format can be optionally set to the returned grammar element.
@@ -47,25 +52,35 @@ def type_from_type(type, format='', format_all=False):
       type: A python type or GrammarType
         A type to be converted to a grammar type (GrammarType class descendant)
 
-      format: str or dict
-        The format to be applied to the resulting class. If dict is given, see 'format_for_type'
+      format
+        The format to be applied to the resulting class. If dict is given, see :func:`format_for_type`
         for the way how the format is determined
 
-      format_all: boolean
+      format_all
         If False (default), the format is not applied, if instance of GrammarType is given as
         the type parameter. Otherwise, a copy of the input type with the applied format is returned
+
+      type_map
   """
-  if isinstance(type, Hashable) and type in type_from_type_map:
+  if isinstance(type, GrammarType):
+     if format_all:
+        type = type.copy()
+        type.format = format_for_type(format, normalize_type(type.numpy_type))
+     return type
+
+  if isinstance(type, Hashable):
     type = normalize_type(type)
+    if type in type_map:
+       type = type_map[type]
+    elif type in type_from_type_map:
+       type = type_from_type_map[type]
+    else:
+       return type
+
     format = format_for_type(format, type)
-    type = type_from_type_map[type]
     if format:
         type = type.copy()
         type.format = format
-    return type
-  elif format_all:
-    type = type.copy()
-    type.format = format_for_type(format, normalize_type(type.numpy_type))
   return type
 
 
@@ -83,38 +98,37 @@ class GrammarType:
       in the parse result. On the other hand, has_value = False is in the file, but
       not in the result.
 
-        """
-  has_value = True
-
-  """ Default value for BaseValueDefinition.name_in_grammar.
-      Some types (e.g. Tables) commonly have no name (are identified
-      by its position in the potential file.
-  """
-  name_in_grammar = True
-
-  """ Default value for the given type. It can be overriden in the constructor (or just by setting
-  the instantiated object attribute) """
-  default_value = None
-
-  """ Deafault type for creating numpy arrays (e.g. by Table) is object - to be redefined
-      in the descendatns.
-
-      The functions called during...
-      ------------------------------
+      **The functions called during...**
 
       User input:  convert, validate
 
       Output: string -> _string
 
-      Parsing: parse -> ( <_grammar parse actions>, validate(parse_check = True) )
+      Parsing: parse -> ( <_grammar parse actions>, validate(why='parse') )
+
   """
+
+  has_value = True
+
+  name_in_grammar = True
+  """ Default value for ValueDefinition.name_in_grammar.
+      Some types (e.g. Tables) commonly have no name (are identified
+      by its position in the potential file) -- such type could redefine
+      this class property."""
+
+  default_value = None
+  """ Default value for the given type. It can be overriden for particular instances
+  in the constructor (or just by setting the attribute of an instantiated object). """
+
   numpy_type = object
+  """ The numpy dtype of the array, that contains values of this type (see e.g. :class:`Array`).
+      The default type ``object`` can and should be redefined in the descendatns. """
 
   def __init__(self, prefix:Union[str,None]=None, postfix:Union[str,None]=None,
                      format:str='', default_value:Any=None,
                      condition:Union[Callable[[Any], Union[bool,str]],None]=None,
                      after_convert:Union[Callable[[Any], Any],None]=None,
-                     additional_description=''):
+                     description=''):
       """
       Create the object.
 
@@ -159,7 +173,7 @@ class GrammarType:
       """ Some subclasses has default_value defined via read-only property. """
       if default_value is not None:
          self.default_value = self.convert(default_value)
-      self._additional_description = additional_description
+      self._description = description
 
   def __str__(self):
       return self.__class__.__name__
@@ -174,11 +188,23 @@ class GrammarType:
     return a == b
 
   @cache
-  def grammar(self, param_name=False):
-    """ Return a pyparsing grammar for the type """
+  def grammar(self, param_name:str=False):
+    """ Return a pyparsing grammar for the type
+
+    Parameters
+    ----------
+
+    param_name
+      The name of the value, that can be assigned to the generated grammar element.
+    """
     grammar = self._grammar
-    if not isinstance(grammar, pp.ParserElement):
+
+    if isinstance(self._grammar, pp.ParserElement):
+       grammar = pp.Forward()
+       grammar << self._grammar
+    else:
        grammar = grammar(param_name)
+
     if self.prefix or self.postfix:
        with generate_grammar():
         if self.prefix:
@@ -190,7 +216,7 @@ class GrammarType:
     if self.has_value:
        def validate(s, loc, x):
            try:
-             out = self.validate(x[0], parse_check=True, param_name=param_name)
+             out = self.validate(x[0], why='parse', param_name=param_name)
            except ValueError as e:
              raise pp.ParseException(s, loc, str(e) + '\nValidating of the parsed value failed') from e
            return x
@@ -237,7 +263,7 @@ class GrammarType:
     """
     return False, None, None
 
-  def validate(self, value, param_name='<Unknown>', parse_check=False):
+  def validate(self, value, param_name='<Unknown>', why:str='set'):
     """ Validate either the pyparsing result or a user given value.
 
     Do not override this method in subclasses for the validation implementation,
@@ -250,9 +276,16 @@ class GrammarType:
     param_name : str or callable
       Parameter name to be used in possible throwed exception (Optional)
       If it is callable, it should be a function that returns the param_name
+    why
+      Possible values are:
+      ``set`` - validation value setted by user (in rare cases, such value can be incomplete
+                and requires `completing` during ``set_from_atoms`` call before saving the output)
+      ``parse`` - validation during parsing input file, checks enforced
+                  by the grammar can be skipped
+      ``save`` - validation before saving the values
     """
     try:
-      err = self._validate(value, parse_check)
+      err = self._validate(value, why)
     except ValueError as err:
       self._valueError(value, err, param_name)
     if err is not True:
@@ -263,7 +296,7 @@ class GrammarType:
         self._valueError(value, err, param_name)
     return True
 
-  def _validate(self, value, parse_check=False):
+  def _validate(self, value, why='set'):
     """ Return error message if the value is not valid. """
     return True
 
@@ -340,47 +373,149 @@ class GrammarType:
     additional_description
       The additional description (e.g. possible choices) of the type. Multiline string.
     """
-    out = self._additional_description
-    if prefix:
-       out.replace('\n', '\n' + prefix)
+    out = self._description
+    if prefix and out:
+       out = out.replace('\n', '\n' + prefix)
     return out
 
-class Unsigned(GrammarType):
+  def type_validation(self, value, types:Union[List[Type], Type], typename:Optional[str]=None):
+    """
+    Parameters
+    ----------
+    value: mixed
+      Value to be checked
+
+    types
+      The required type or types. If more types is given, it is sufficient, if the value is of
+      any of given types.
+
+    Returns
+    -------
+    error_message: Union[str, bool]
+      The function returns either False, if the value is ok, or string containing an error
+      message describing the error.
+
+    """
+    if isinstance(value, types): return True
+    if not typename:
+       typename = types
+    return f"An {typename} value is required, a value {value} of type {value.__class__} have been given"
+
+@add_called_class_as_argument
+def add_to_parent_validation(validation):
+
+    @functools.wraps(validation)
+    def wrapped(cls, self, value, why='set'):
+        out = super(cls, self)._validate(value, why)
+        if out is not True:
+           return out
+        return validation(self, value, why)
+
+    return wrapped
+
+
+class TypedGrammarType(GrammarType):
+
+  @cached_class_property
+  def datatype(cls):
+      """ The (primary) type of the value. Redefine it in the descendants, if it is needed. """
+      return cls.numpy_type
+
+  @cached_class_property
+  def allowed_types(cls):
+      """ Allowed alternative types, that will be converted to the 'primary' datatype. """
+      dt = cls.datatype
+      return allowed_types.get(dt, (dt, ))
+
+  def convert(self, value):
+      if isinstance(value, self.datatype):
+         return value
+      for i in self.allowed_types:
+          if isinstance(value, i):
+             return self.datatype(value)
+      return value
+
+  @cached_class_property
+  def datatype_name(cls):
+      return cls.__name__.lower()
+
+  def _validate(self, value, why='set'):
+      return self.type_validation(value, self.allowed_types, self.datatype_name)
+
+class Number(TypedGrammarType):
+  """ Base class for a number - descendants of this class can have minimal and/or maximal possible value. """
+
+  @add_to_signature(GrammarType.__init__)
+  def __init__(self, min:Optional[int]=None, max:Optional[int]=None, *args, **kwargs):
+      """
+      Parameters
+      ----------
+      min:
+        Minimal allowed value.
+
+      max:
+        Maximal allowed value.
+      """
+      self.min = min
+      self.max = max
+      super().__init__(*args, **kwargs)
+
+  @add_to_parent_validation
+  def _validate(self, value, why='set'):
+      if self.min is not None and self.min > value:
+         return f"A value greater that or equal to {self.min} is required, {value} have been given."
+      if self.max is not None and self.max < value:
+         return f"A value less than or equal to {self.max} is required, {value} have been given."
+      return True
+
+
+class Unsigned(Number):
   """ Unsigned integer (zero is possible) """
 
   _grammar = replace_whitechars(ppc.integer).setParseAction(lambda x:int(x[0]))
 
-  def _validate(self, value, parse_check=False):
-    if not isinstance(value, int): return "An integer value required"
-    return value >= 0 or "A positive value required"
+  @add_to_parent_validation
+  def _validate(self, value, why='set'):
+      return value >= 0 or "A positive value required"
 
   def grammar_name(self):
     return '<+int>'
 
   numpy_type = int
+  datatype_name = 'unsigned integer'
 
+class ObjectNumber(Unsigned):
+  """ An abstract class, that describe an unsigned integer, that reffers to an object.
+  User can give the object either using the object, or by the number. Descendant classes
+  should take care of transforming the object to the resulting integer (by setting
+  the result property of the described :class:`Option<ase2sprkkr.common.options.Option>`)
 
-class Integer(GrammarType):
+  The type of te object should be given by the type class property.
+  """
+
+  def convert(self, value):
+      if isinstance(value, self.type):
+         return value
+      super().convert()
+
+  def _validate(self, value, why='set'):
+      return isinstance(value, self.type) or super.validate()
+
+class Integer(Number):
   """ Signed integer """
 
   _grammar = replace_whitechars(ppc.signed_integer).setParseAction(lambda x:int(x[0]))
 
-  def _validate(self, value, parse_check=False):
-    return isinstance(value, (int, np.int64) ) or f'An integer value required, ({value.__class__} was given)'
-
   def grammar_name(self):
     return '<int>'
 
   numpy_type = int
 
 
-class Bool(GrammarType):
+class Bool(TypedGrammarType):
   """ A bool type, whose value is represented by a letter (T or F) """
   _grammar = (pp.CaselessKeyword('T') | pp.CaselessKeyword('F')).setParseAction( lambda x: x[0] == 'T' )
 
-  def _validate(self, value, parse_check=False):
-    return isinstance(value, bool) or "A bool value rquired"
-
   def grammar_name(self):
     return '<T|F>'
 
@@ -388,38 +523,34 @@ class Bool(GrammarType):
     return 'T' if val else 'F'
 
   numpy_type = bool
+  type_name = 'boolean'
 
 
-class Real(GrammarType):
+class Real(Number):
   """ A real value """
   _grammar = replace_whitechars(ppc.fnumber).setParseAction(lambda x: float(x[0]))
 
-  def _validate(self, value, parse_check=False):
-    return isinstance(value, float) or "A float value required"
-
   def grammar_name(self):
     return '<float>'
 
   numpy_type = float
 
 
-class Date(GrammarType):
+class Date(Number):
   """ A date value of the form 'DD.MM.YYYY' """
 
   _grammar = pp.Regex(r'(?P<d>\d{2}).(?P<m>\d{2}).(?P<y>\d{4})').setParseAction(lambda x: datetime.date(int(x['y']), int(x['m']), int(x['d'])))
 
-  def _validate(self, value, parse_check=False):
-    return isinstance(value, datetime.date) or "A date (datetime.date) value required"
-
   def grammar_name(self):
     return '<dd.mm.yyyy>'
 
   def _string(self, val):
     return val.strftime("%d.%m.%Y")
 
+  numpy_type = datetime.date
+  type_name = 'date'
 
-
-class BaseRealWithUnits(GrammarType):
+class BaseRealWithUnits(Real):
   """ The base class for float value, which can have units append.
       The value is converted automatically to the base units.
   """
@@ -443,7 +574,7 @@ class BaseRealWithUnits(GrammarType):
   def _grammar(self, param_name):
     return self._grammar_units(self.units)
 
-  def _validate(self, value, parse_check=False):
+  def _validate(self, value, why='set'):
     return isinstance(value, float) or "A float value required"
 
   def grammar_name(self):
@@ -471,16 +602,19 @@ class Energy(BaseRealWithUnits):
   def __str__(self):
       return "Energy (<Real> [Ry|eV])"
 
-class BaseString(GrammarType):
+class BaseString(TypedGrammarType):
   """ Base type for string grammar types """
 
-  def _validate(self, value, parse_check=False):
-    if not isinstance(value, str): return "A string value required"
-    if not parse_check:
+  datatype = str
+  datatype_name = 'string'
+
+  @add_to_parent_validation
+  def _validate(self, value, why='set'):
+    if not why=='parse':
       try:
         self._grammar.parseString(value, True)
       except pp.ParseException as e:
-        return f"Forbidden character '{e.line[e.col]}' in the string"
+        return f"Forbidden character '{e.line[e.col-1]}' in the string"
     return True
 
 class String(BaseString):
@@ -524,7 +658,7 @@ class Keyword(GrammarType):
     with generate_grammar():
       self._grammar = optional_quote + pp.MatchFirst((pp.CaselessKeyword(i) for i in self.keywords)).setParseAction(lambda x: x[0].upper()) + optional_quote
 
-  def _validate(self, value, parse_check=False):
+  def _validate(self, value, why='set'):
     return value in self.keywords or "Required one of [" + "|".join(self.keywords) + "]"
 
   def grammar_name(self):
@@ -542,10 +676,10 @@ class Keyword(GrammarType):
       ad = super().additional_description(prefix)
       if not self.choices:
          return ad
-      out = f'{prefix}Possible values:\n'
+      out = f'\n{prefix}Possible values:\n'
       out += '\n'.join([f"{prefix}  {k:<10}{v}" for k,v in self.choices.items()])
       if ad:
-         out += '\n' + ad
+         out += f'\n\n{prefix}' + ad
       return out
 
 
@@ -553,14 +687,20 @@ def DefKeyword(default, *others, **kwargs):
   """
   A value, that can take values from the predefined set of strings, the first one is the default value.
   """
-  return Keyword(default, *others, default_value=default, **kwargs)
+  if isinstance(default, dict) and len(others) == 0:
+     def_val = next(iter(default))
+  else:
+     def_val = default
+  return Keyword(default, *others, default_value=def_val, **kwargs)
 
 
-class Flag(GrammarType):
+class Flag(TypedGrammarType):
   """
   A boolean value, which is True, if a name of the value appears in the input file.
   """
 
+  numpy_type = bool
+
   def grammar_name(self):
       return None
 
@@ -570,7 +710,7 @@ class Flag(GrammarType):
   def missing_value(self):
       return (True, True, False)
 
-  def _validate(self, value, parse_check=False):
+  def _validate(self, value, why='set'):
       return value is True or value is False or value is None or "This is Flag with no value, please set to True to be present or to False/None to not"
 
   _grammar = pp.Empty().setParseAction(lambda x: True)
@@ -584,7 +724,7 @@ class Array(GrammarType):
 
   def __init__(self, type, default_value=None,
                length=None, max_length=None, min_length=None,
-               as_list=False,
+               as_list=False, format=None,
                **kwargs):
     """
     Parameters
@@ -614,6 +754,9 @@ class Array(GrammarType):
         default_value = type
         type = type[0].__class__
     self.type = type_from_type(type)
+    if format is not None:
+       self.type = self.type.copy()
+       self.type.format = format
     self.as_list = as_list
     super().__init__(default_value=default_value, **kwargs)
     self.min_length = min_length or length
@@ -666,7 +809,7 @@ class Array(GrammarType):
        out += self.type.string(i)
     return out
 
-  def _validate(self, value, parse_check=False):
+  def _validate(self, value, why='set'):
     if callable(self.as_list):
        cls = self.as_list
     elif self.as_list:
@@ -678,7 +821,7 @@ class Array(GrammarType):
 
     for i,v in enumerate(value):
         try:
-          self.type.validate(v, parse_check=False)
+          self.type.validate(v, why='set')
         except ValueError as e:
            raise ValueError("Value {} in the set is incorrect: {}".format(i, str(e))) from e
     if self.min_length is not None and len(value) < self.min_length:
@@ -729,7 +872,7 @@ class Complex(SetOf):
   def convert(self, value):
     return complex(value)
 
-  def _validate(self, value, parse_check=False):
+  def _validate(self, value, why='set'):
     return isinstance(value, (complex, np.complexfloating)) or 'A complex value required, {value} given.'
 
   def _grammar_name(self):
@@ -740,7 +883,7 @@ class Complex(SetOf):
 
   __str__ = GrammarType.__str__
 
-def type_from_value(value):
+def type_from_value(value, type_map={}):
   """ Gues the grammar type from a python value.
 
   ..doctest::
@@ -758,12 +901,12 @@ def type_from_value(value):
         return String.I
      except Exception:
         return QString.I
-  type = type_from_type(value.__class__)
+  type = type_from_type(value.__class__, type_map=type_map)
   if type is value.__class__:
      raise ValueError(f'Cannot determine grammar type from value {value}')
   return type.__class__(default_value = value)
 
-def type_from_default_value(value, format='', format_all=False):
+def type_from_default_value(value, format='', format_all=False, type_map={}):
    """ Guess the grammar type from a value, that will become the default value of the grammar type.
 
    It has to create a new object instance, as it has to set the default
@@ -773,9 +916,9 @@ def type_from_default_value(value, format='', format_all=False):
    Grammar types passed as types are left as is, unless format_all flag is set.
    """
    if inspect.isclass(value) or isinstance(value, GrammarType):
-      return type_from_type(value, format=format, format_all=format_all)
+      return type_from_type(value, format=format, format_all=format_all, type_map={})
    ptype = normalize_type(value.__class__)
-   gtype = type_from_type(value.__class__).__class__
+   gtype = type_from_type(value.__class__, type_map=type_map).__class__
    return gtype(default_value = value, format=format_for_type(format, ptype))
 
 class BaseMixed(GrammarType):
@@ -801,16 +944,20 @@ class BaseMixed(GrammarType):
       """
       return self.string_type if isinstance(value, str) else type_from_value(value)
 
-  def _validate(self, value, parse_check=False):
+  def _validate(self, value, why='set'):
+      if value is None:
+          return True
       type = self.get_type(value)
       if type is value:
-         return 'Can not determine the type of value {}'.format(value)
-      return type.validate(value, parse_check)
+          return 'Can not determine the type of value {}'.format(value)
+      return type.validate(value, why)
 
   def grammar_name(self):
       return '<mixed>'
 
   def convert(self, value):
+      if value is None:
+          return None
       return self.get_type(value).convert(value)
 
 class Range(BaseMixed):
@@ -889,7 +1036,7 @@ class Separator(GrammarType):
   _grammar = separator_grammar.copy().setParseAction(lambda x: [None])
   has_value = False
 
-  def _validate(self, value, parse_check=False):
+  def _validate(self, value, why='set'):
       return 'Can not set a value to a separator'
 
   def _grammar_name(self):
@@ -934,11 +1081,11 @@ class Sequence(GrammarType):
          grammar.addConditionEx(lambda x: x[0] in self.allowed_values, lambda x: f'{x[0]} is not in the list of allowed values')
       return grammar
 
-  def _validate(self, value, parse_check=False):
+  def _validate(self, value, why='set'):
       if not isinstance(value, (self.value_type)) or len(value) != len(self.types):
           return f'A tuple of {len(self.types)} values is required'
       for i,j in zip(self.types, value):
-          out = i.validate(j, parse_check=parse_check)
+          out = i.validate(j, why=why)
       return True
 
   def convert(self, value):
@@ -1107,7 +1254,7 @@ class Table(GrammarType):
          out.append(self.sequence.string(i))
       return ''.join(out)
 
-  def _validate(self, value, parse_check=False):
+  def _validate(self, value, why='set'):
       if not isinstance(value, np.ndarray):
          return f"Numpy array as a value required {value.__class__} given"
       dtype = self.numpy_type
@@ -1205,26 +1352,6 @@ mixed = Mixed.I = Mixed()
 pot_mixed = PotMixed.I = PotMixed()
 """ A standard grammar type instance for variant (mixed) values in potential files """
 
-# some mapping and other stuff used by the types
-normalize_type_map = {
-    np.int64 : int,
-    np.float64: float,
-    np.complex128 : complex,
-    np.bool_: bool
-}
-""" Mapping of alternative types to the 'canonical ones'. """
-
-def normalize_type(type):
-    """ Return the 'canonical type' for a given type.
-
-    I.e. it maps numpy internal types to standard python ones.
-
-    doctest:
-    >>> normalize_type(np.int64)
-    <class 'int'>
-    """
-    return normalize_type_map.get(type, type)
-
 type_from_type_map = OrderedDict([
     (float, Real.I),
     (complex, Complex.I),
