@@ -3,10 +3,9 @@ from ...common.grammar_types  import *
 from .sections import *
 from ..input_parameters_definitions import \
     InputParametersDefinition as InputParameters, \
-    ValueDefinition as V, \
-    SectionDefinition as Section
+    InputValueDefinition as V, \
+    InputSectionDefinition as Section
 
-#: ARPES task input parameters definition
 input_parameters = InputParameters(
     'arpes', [
     CONTROL('ARPES'),
@@ -81,7 +80,7 @@ input_parameters = InputParameters(
                         2: "scattering-angle diagram -> theta scan",
                         3: "orthonormal projection",
                         4: "stereographic projection"},
-                      additional_description = '3,4 only for angular resolved\npe (ups, xps) note: nt=np-> nx,ny'
+                       description = '3,4 only for angular resolved\npe (ups, xps) note: nt=np-> nx,ny'
               ), expert=1,
               info='Crystal coordinats in splout, xpsrun, or upsrun'),
       V('ISTR', Array(int, length=2), expert=[0,0], info="beam number (h,k)"),
@@ -105,3 +104,6 @@ input_parameters = InputParameters(
   info = 'ARPES - angle resolved photoemission spectroscopy'
 )
 """ ARPES task input parameters definition"""
+
+from ...common.doc import process_input_parameters_definition
+process_input_parameters_definition(__name__)
