@@ -3,7 +3,7 @@ from ...common.grammar_types  import *
 from .sections import *
 from ..input_parameters_definitions import \
     InputParametersDefinition as InputParameters, \
-    ValueDefinition as V \
+    InputValueDefinition as V \
 
 input_parameters = InputParameters(
       'dos', [
@@ -23,3 +23,6 @@ input_parameters = InputParameters(
   mpi=True
 )
 """ DOS task input parameters definition"""
+
+from ...common.doc import process_input_parameters_definition
+process_input_parameters_definition(__name__)
