@@ -3,7 +3,7 @@ from ...common.grammar_types  import *
 from .sections import *
 from ..input_parameters_definitions import \
     InputParametersDefinition as InputParameters, \
-    ValueDefinition as V
+    InputValueDefinition as V
 
 input_parameters = InputParameters(
       'scf', [
@@ -13,11 +13,17 @@ input_parameters = InputParameters(
         TAU,
         ENERGY,
         SCF,
-        SITES
+        SITES,
+        STRCONST,
+        CPA,
+        MODE
       ],
       info = "SCF - calculate a self-consistent potential",
-      description = "<TODO: fill something meaningfull>",
+      description = "",
       executable = 'kkrscf',
       mpi = True
 )
 """ SCF task input parameters definition"""
+
+from ...common.doc import process_input_parameters_definition
+process_input_parameters_definition(__name__)
