@@ -11,15 +11,14 @@ import functools
 import pyparsing as pp
 from ..common.grammar import line_end
 from ..common.configuration_definitions import \
-    BaseValueDefinition, \
-    BaseSectionDefinition
-from ..common.misc import lazy_value
+    ValueDefinition, \
+    SectionDefinition
+from ..common.decorators import cached_class_property
 
-class OutputValueDefinition(BaseValueDefinition):
+class OutputValueDefinition(ValueDefinition):
   """ Value in an output file, of a form 'NAME   VALUE' """
 
-  @staticmethod
-  @lazy_value
+  @cached_class_property
   def grammar_of_delimiter():
     return pp.WordStart()
 
@@ -33,8 +32,7 @@ class OutputValueEqualDefinition(OutputValueDefinition):
 
   name_value_delimiter = '='
 
-  @staticmethod
-  @lazy_value
+  @cached_class_property
   def grammar_of_delimiter():
     return pp.Suppress("=").setName('=')
 
@@ -49,8 +47,7 @@ class OutputNonameValueDefinition(OutputValueDefinition):
       super().__init__(*args, required=required, **kwargs)
 
 
-
-class OutputSectionDefinition(BaseSectionDefinition):
+class OutputSectionDefinition(SectionDefinition):
   """ This class describes the format of one
   value of a standard potential section """
 
@@ -69,8 +66,7 @@ class OutputSectionDefinition(BaseSectionDefinition):
   delimiter = '\n'
   """ options are delimited by newline in ouptut. """
 
-  @staticmethod
-  @lazy_value
+  @cached_class_property
   def grammar_of_delimiter():
       out = (pp.Optional(line_end) + pp.WordStart()).suppress()
       return out
